package squarematrix;

/**
 * Model a square matrix.
 * 
 * Experiment with matrix multiplication to
 * show the convergence seen in the case study
 * in Section 1.6 of the textbook.
 * 
 * <a href="http://mathworld.wolfram.com/MatrixMultiplication.html">
 * Read more about matrix multiplication.</a>
 * 
 * @author Your Name
 * @version 23 February 2015
 */
public class SquareMatrix {

    /**
     * The dimension is the number of rows and
     * columns in this square matrix.
     */
    private final int dimension;
    /**
     * This matrix stores the floating point values of its
     * elements in a two dimensional array.
     */
    private final double[][] elements;

    /**
     * Create a square matrix that has a given number of
     * rows and columns, with all elements equal to zero.
     * 
     * @param dimension is the number of rows and columns.
     */
    public SquareMatrix(int dimension) {
        this.dimension = dimension;
        this.elements = new double[dimension][dimension];
        for (int i = 0; i < dimension; i++) {
            for (int j = 0; j < dimension; j++) {
                this.setElement(i, j, 0.0);
            } // for
        } // for
    } // SquareMatrix( int )

    /**
     * Retrieve the dimension of this matrix.
     * 
     * @return the number of rows and columns that
     * this square matrix has.
     */
    public final int getDimension() {
        // Need more code here.
        // (Replace 0 in return statement with something else.)
        return 0;
    } // getDimension();

    /**
     * Retrieve the value of the element of this
     * matrix that is at the specified position.
     * 
     * @param row
     * @param column
     * @return the value at (row, column).
    */
    public final double getElement(int row, int column) {
        // Need more code here.
        // (Replace 0.0 in return statement with something else.)
        return 0.0;
    } // getElement( int, int )

    /**
     * Change the value of a specified element of this
     * matrix.
     * 
     * @param row
     * @param column
     * @param value is the new value to be assigned to the element
     * at (row, column).
     */
    public final void setElement(int row, int column, double value) {
        // Need more code here.
    } // setElement( int, int, double )

    /**
     * Create a printable representation of this matrix.
     * 
     * @return a string that contains newline characters
     * at the end of each row.
     */
    @Override
    public String toString() {
        StringBuilder result = new StringBuilder();
        for( int i = 0; i < this.getDimension(); i++ ) {
            for( int j = 0; j < this.getDimension(); j++ ) {
                String formattedString = String.format( "%6.2f ", 
                        this.getElement(i,j));
                result.append( formattedString );
            } // for
            result.append( "\n" );
        } // for
        return result.toString();
    } // toString()
    
    /**
     * Create another matrix that is equal to this one.
     * 
     * "Equal" means that for every row i and column j
     * the elements at (i,j) in each matrix are equal.
     * 
     * @return a copy of this matrix 
     */
    public SquareMatrix copy() {
        SquareMatrix result = new SquareMatrix( this.getDimension() );
        for( int i = 0; i < this.getDimension(); i++ ) {
            for( int j = 0; j < this.getDimension(); j++ ) {
                result.setElement(i, j, this.getElement(i,j));
            } // for
        } // for
        return result;
    } // copy()
    
    /**
     * Set the values of the elements of this matrix
     * to make an identity matrix.
     * 
     * In an identity matrix, the values of all elements
     * that are on the main diagonal (upper left to lower right)
     * are one and the values of all other elements are zero.
     */
    public void setToIdentity() {
       // Need more code here.
    } // setToIdentity()
    
    /**
     * Set the values of the elements of this matrix
     * to make rows that represent PDFs.
     * 
     * A row represents a PDF if all values within
     * the row are in the interval [0,1) and the sum
     * of all values is 1.0.
     */
    public void setToPDFs() {
        for (int i = 0; i < this.getDimension(); i++) {
            // fill row with random numbers between 0.0 and 1.0
            for (int j = 0; j < this.dimension; j++) {
                this.setElement(i, j, Math.random());
            } // for

            // find sum of numbers in row i
            double sum = 0.0;
            for (int j = 0; j < this.getDimension(); j++) {
                sum += this.getElement(i, j);
            } // for

            // divide each number in row i by sum
            for (int j = 0; j < this.getDimension(); j++) {
                double e = this.getElement(i, j);
                this.setElement(i, j, e / sum);
            } // for
        } // for
    } // setToPDFs()

    /**
     * Create a new matrix that is the product
     * of this matrix and another given matrix.
     * 
     * @param m is the other matrix.
     * @return the product of this matrix and m.
     */
    public SquareMatrix multiply(SquareMatrix m) {
        int n = this.getDimension();
        if (n != m.getDimension()) {
            String message = "Matrices must have the same dimension.";
            throw new IllegalArgumentException(message);
        } // if

        SquareMatrix productOfMatrices = new SquareMatrix(n);

        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                double sumOfProducts = 0.0;
                for (int k = 0; k < n; k++) {
                    // Need more code here.
                } // for
                productOfMatrices.setElement(i, j, sumOfProducts);
            } // for
        } // for
        return productOfMatrices;
    } // multiply( SquareMatrix, SquareMatrix )

    /**
     * Compute the product of this matrix with itself.
     * 
     * @return the square of this matrix. 
     */
    public SquareMatrix square() {
        return this.multiply(this);
    } // square()
    
    /**
     * Compute a matrix that is equal to this matrix
     * raised to a given integer power.
     * 
     * @param exponent is the power to which the method
     * raises this matrix.
     * @return the product of this matrix multiplied times itself
     * the specified number of times.
     */
    public SquareMatrix power( int exponent ) {
        SquareMatrix result = new SquareMatrix( this.getDimension() );
        
        if( exponent == 0 ) {
            result.setToIdentity();
        } // if
        else if( exponent == 1 ) {
            result = this.copy();
        } // else if
        else {
            SquareMatrix smallerPower = this.power(exponent/2);
            result = smallerPower.square();
            if( exponent % 2 == 1 ) {
                result = result.multiply(this);
            } // if
        } // else 

        return result;
    } // power( int )
    
    public static void main(String[] args) {
        SquareMatrix m = new SquareMatrix(4);
        //m.setToPDFs();
        System.out.println( m );
        
        SquareMatrix m2 = m.power(2);
        System.out.println( m2 );
        
        SquareMatrix m4 = m.power(4);
        System.out.println( m4 );
        
        SquareMatrix m8 = m.power(8);
        System.out.println( m8 );
    } // main( String [] )

} // SquareMatrix
