package squarematrix;

import java.util.Scanner;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import org.junit.Test;

public class SquareMatrixTest {

    private static final double EPSILON = 1E-8;

    /**
     * Test of getDimension method, of class SquareMatrix.
     */
    @Test
    public void testGetDimension() {
        SquareMatrix instance = new SquareMatrix(4);
        int expResult = 4;
        int result = instance.getDimension();
        assertEquals(expResult, result);
    } // testGetDimension()

    /**
     * Test of getElement method, of class SquareMatrix.
     */
    @Test
    public void testGetElement() {
        int n = 4;
        SquareMatrix instance = new SquareMatrix(n);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                int k = i * n + j;
                instance.setElement(i, j, k);

                double expResult = k;
                double result = instance.getElement(i, j);
                assertEquals(expResult, result, EPSILON);
            } // for
        } // for
    } // testGetElement()

    /**
     * Test of setElement method, of class SquareMatrix.
     */
    @Test
    public void testSetElement() {
        int n = 4;
        SquareMatrix instance = new SquareMatrix(n);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                int k = i * n + j;
                instance.setElement(i, j, k);

                double expResult = k;
                double result = instance.getElement(i, j);
                assertEquals(expResult, result, EPSILON);
            } // for
        } // for
    } // testSetElement()

    @Test
    public void TestToString() {
        int n = 4;
        SquareMatrix instance = new SquareMatrix(n);
        String printableRepresentation = instance.toString();
        Scanner scanner = new Scanner( printableRepresentation );
        while( scanner.hasNext() ) {
           double x = scanner.nextDouble();
           assertEquals( x, 0.0, EPSILON );
        } // while
    } // TestToString()
    
    /**
     * Test of copy method, of class SquareMatrix.
     */
    @Test
    public void testCopy() {
        int n = 4;
        SquareMatrix instance = new SquareMatrix(n);
        SquareMatrix copy = instance.copy();
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                double u = instance.getElement(i, j);
                double v = copy.getElement(i, j);
                assertEquals(u, v, EPSILON);
            } // for
        } // for
    } // testCopy()

    /**
     * Test of setToIdentity method, of class SquareMatrix.
     */
    @Test
    public void testSetToIdentity() {
        int n = 4;
        SquareMatrix instance = new SquareMatrix(n);
        instance.setToIdentity();
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                double x = instance.getElement(i, j);
                if (i == j) {
                    assertEquals(x, 1.0, EPSILON);
                } // if
                else {
                    assertEquals(x, 0.0, EPSILON);
                } // else
            } // for
        } // for
    } // testSetToIdentity()

    /**
     * Test of setToPDFs method, of class SquareMatrix.
     */
    @Test
    public void testSetToPDFs() {
        int n = 4;
        SquareMatrix instance = new SquareMatrix(n);
        instance.setToPDFs();
        for (int i = 0; i < n; i++) {
            double sum = 0.0;
            for (int j = 0; j < n; j++) {
                double x = instance.getElement(i, j);
                assertTrue(0.0 <= x && x < 1.0);
                sum += x;
            } // for
            assertEquals(sum, 1.0, EPSILON);
        } // for
    } // testSetToPDFs()

    /**
     * Test of multiply method, of class SquareMatrix.
     */
    @Test
    public void testMultiply() {
        SquareMatrix a = new SquareMatrix(2);
        double cos30 = Math.cos(Math.PI / 6);
        double sin30 = Math.sin(Math.PI / 6);
        a.setElement(0, 0, cos30);
        a.setElement(0, 1, -sin30);
        a.setElement(1, 0, sin30);
        a.setElement(1, 1, cos30);

        SquareMatrix b = new SquareMatrix(2);
        double cos15 = Math.cos(Math.PI / 12);
        double sin15 = Math.sin(Math.PI / 12);
        b.setElement(0, 0, cos15);
        b.setElement(0, 1, -sin15);
        b.setElement(1, 0, sin15);
        b.setElement(1, 1, cos15);

        SquareMatrix c = a.multiply(b);
        double cos45 = Math.cos(Math.PI / 4);
        double sin45 = Math.sin(Math.PI / 4);
        assertEquals(c.getElement(0, 0), cos45, EPSILON);
        assertEquals(c.getElement(0, 1), -sin45, EPSILON);
        assertEquals(c.getElement(1, 0), sin45, EPSILON);
        assertEquals(c.getElement(1, 1), cos45, EPSILON);
    } // testMultiply()

    /**
     * Test of square method, of class SquareMatrix.
     */
    @Test
    public void testSquare() {
        SquareMatrix a = new SquareMatrix(2);
        double cos30 = Math.cos(Math.PI / 6);
        double sin30 = Math.sin(Math.PI / 6);
        a.setElement(0, 0, cos30);
        a.setElement(0, 1, -sin30);
        a.setElement(1, 0, sin30);
        a.setElement(1, 1, cos30);

        SquareMatrix aSquared = a.square();
        double cos60 = Math.cos(Math.PI / 3);
        double sin60 = Math.sin(Math.PI / 3);
        assertEquals(aSquared.getElement(0, 0), cos60, EPSILON);
        assertEquals(aSquared.getElement(0, 1), -sin60, EPSILON);
        assertEquals(aSquared.getElement(1, 0), sin60, EPSILON);
        assertEquals(aSquared.getElement(1, 1), cos60, EPSILON);
    } // testSquare()

    /**
     * Test of power method, of class SquareMatrix.
     */
    @Test
    public void testPower() {
        int n = 4;
        int exponent = 16;
        SquareMatrix m = new SquareMatrix(n);
        m.setToIdentity();
        SquareMatrix p = m.power(exponent);
        for( int i = 0; i < n; i++ ) {
            for( int j = 0; j < n; j++ ) {
                double x = p.getElement(i, j);
                if( i == j ) {
                    assertEquals( x, 1.0, EPSILON );
                } // if
                else {
                    assertEquals( x, 0.0, EPSILON );
                } // else
            } // for
        } // for
    } // testPower()

} // SquareMatrixTest
